/*******************************************************************************
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 ******************************************************************************/

package org.emftext.language.refactoring.rolemapping.resource.rolemapping.util;

/**
 * This class provides basic infrastructure to interpret models. To implement
 * concrete interpreters, subclass this abstract interpreter and override the
 * interprete_* methods. The interpretation can be customized by binding the two
 * type parameters (ResultType, ContextType). The former is returned by all
 * interprete_* methods, while the latter is passed from method to method while
 * traversing the model. The concrete traversal strategy can also be exchanged.
 * One can use a static traversal strategy by pushing all objects to interpret on
 * the interpretation stack (using addObjectToInterprete()) before calling
 * interprete(). Alternatively, the traversal strategy can be dynamic by pushing
 * objects on the interpretation stack during interpretation.
 */
public class AbstractRolemappingInterpreter<ResultType, ContextType> {
	
	private java.util.Stack<org.eclipse.emf.ecore.EObject> interpretationStack = new java.util.Stack<org.eclipse.emf.ecore.EObject>();
	private java.util.List<org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingInterpreterListener> listeners = new java.util.ArrayList<org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingInterpreterListener>();
	private org.eclipse.emf.ecore.EObject nextObjectToInterprete;
	private Object currentContext;
	
	public ResultType interprete(ContextType context) {
		ResultType result = null;
		org.eclipse.emf.ecore.EObject next = null;
		currentContext = context;
		while (!interpretationStack.empty()) {
			try {
				next = interpretationStack.pop();
			} catch (java.util.EmptyStackException ese) {
				// this can happen when the interpreter was terminated between the call to empty()
				// and pop()
				break;
			}
			nextObjectToInterprete = next;
			notifyListeners(next);
			result = interprete(next, context);
			if (!continueInterpretation(context, result)) {
				break;
			}
		}
		currentContext = null;
		return result;
	}
	
	/**
	 * Override this method to stop the overall interpretation depending on the result
	 * of the interpretation of a single model elements.
	 */
	public boolean continueInterpretation(ContextType context, ResultType result) {
		return true;
	}
	
	public ResultType interprete(org.eclipse.emf.ecore.EObject object, ContextType context) {
		ResultType result = null;
		if (object instanceof org.emftext.language.refactoring.rolemapping.RoleMappingModel) {
			result = interprete_org_emftext_language_refactoring_rolemapping_RoleMappingModel((org.emftext.language.refactoring.rolemapping.RoleMappingModel) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.rolemapping.RoleMapping) {
			result = interprete_org_emftext_language_refactoring_rolemapping_RoleMapping((org.emftext.language.refactoring.rolemapping.RoleMapping) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.rolemapping.ConcreteMapping) {
			result = interprete_org_emftext_language_refactoring_rolemapping_ConcreteMapping((org.emftext.language.refactoring.rolemapping.ConcreteMapping) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.rolemapping.CollaborationMapping) {
			result = interprete_org_emftext_language_refactoring_rolemapping_CollaborationMapping((org.emftext.language.refactoring.rolemapping.CollaborationMapping) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.rolemapping.AttributeMapping) {
			result = interprete_org_emftext_language_refactoring_rolemapping_AttributeMapping((org.emftext.language.refactoring.rolemapping.AttributeMapping) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.rolemapping.ReferenceMetaClassPair) {
			result = interprete_org_emftext_language_refactoring_rolemapping_ReferenceMetaClassPair((org.emftext.language.refactoring.rolemapping.ReferenceMetaClassPair) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.EAttribute) {
			result = interprete_org_eclipse_emf_ecore_EAttribute((org.eclipse.emf.ecore.EAttribute) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.EAnnotation) {
			result = interprete_org_eclipse_emf_ecore_EAnnotation((org.eclipse.emf.ecore.EAnnotation) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.EClass) {
			result = interprete_org_eclipse_emf_ecore_EClass((org.eclipse.emf.ecore.EClass) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.EEnum) {
			result = interprete_org_eclipse_emf_ecore_EEnum((org.eclipse.emf.ecore.EEnum) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.EDataType) {
			result = interprete_org_eclipse_emf_ecore_EDataType((org.eclipse.emf.ecore.EDataType) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.EClassifier) {
			result = interprete_org_eclipse_emf_ecore_EClassifier((org.eclipse.emf.ecore.EClassifier) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.EEnumLiteral) {
			result = interprete_org_eclipse_emf_ecore_EEnumLiteral((org.eclipse.emf.ecore.EEnumLiteral) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.EFactory) {
			result = interprete_org_eclipse_emf_ecore_EFactory((org.eclipse.emf.ecore.EFactory) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.ENamedElement) {
			result = interprete_org_eclipse_emf_ecore_ENamedElement((org.eclipse.emf.ecore.ENamedElement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.EModelElement) {
			result = interprete_org_eclipse_emf_ecore_EModelElement((org.eclipse.emf.ecore.EModelElement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.EObject) {
			result = interprete_org_eclipse_emf_ecore_EObject((org.eclipse.emf.ecore.EObject) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.EOperation) {
			result = interprete_org_eclipse_emf_ecore_EOperation((org.eclipse.emf.ecore.EOperation) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.EPackage) {
			result = interprete_org_eclipse_emf_ecore_EPackage((org.eclipse.emf.ecore.EPackage) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.EParameter) {
			result = interprete_org_eclipse_emf_ecore_EParameter((org.eclipse.emf.ecore.EParameter) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.EReference) {
			result = interprete_org_eclipse_emf_ecore_EReference((org.eclipse.emf.ecore.EReference) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.EStructuralFeature) {
			result = interprete_org_eclipse_emf_ecore_EStructuralFeature((org.eclipse.emf.ecore.EStructuralFeature) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.ETypedElement) {
			result = interprete_org_eclipse_emf_ecore_ETypedElement((org.eclipse.emf.ecore.ETypedElement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof java.util.Map.Entry<?,?>) {
			result = interprete_java_util_Map_Entry((java.util.Map.Entry<?,?>) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.EGenericType) {
			result = interprete_org_eclipse_emf_ecore_EGenericType((org.eclipse.emf.ecore.EGenericType) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.eclipse.emf.ecore.ETypeParameter) {
			result = interprete_org_eclipse_emf_ecore_ETypeParameter((org.eclipse.emf.ecore.ETypeParameter) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.roles.NamedElement) {
			result = interprete_org_emftext_language_refactoring_roles_NamedElement((org.emftext.language.refactoring.roles.NamedElement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.roles.Collaboration) {
			result = interprete_org_emftext_language_refactoring_roles_Collaboration((org.emftext.language.refactoring.roles.Collaboration) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.roles.Role) {
			result = interprete_org_emftext_language_refactoring_roles_Role((org.emftext.language.refactoring.roles.Role) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.roles.RoleModel) {
			result = interprete_org_emftext_language_refactoring_roles_RoleModel((org.emftext.language.refactoring.roles.RoleModel) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.roles.Multiplicity) {
			result = interprete_org_emftext_language_refactoring_roles_Multiplicity((org.emftext.language.refactoring.roles.Multiplicity) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.roles.MultiplicityCollaboration) {
			result = interprete_org_emftext_language_refactoring_roles_MultiplicityCollaboration((org.emftext.language.refactoring.roles.MultiplicityCollaboration) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.roles.RoleProhibition) {
			result = interprete_org_emftext_language_refactoring_roles_RoleProhibition((org.emftext.language.refactoring.roles.RoleProhibition) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.roles.RoleAssociation) {
			result = interprete_org_emftext_language_refactoring_roles_RoleAssociation((org.emftext.language.refactoring.roles.RoleAssociation) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.roles.RoleComposition) {
			result = interprete_org_emftext_language_refactoring_roles_RoleComposition((org.emftext.language.refactoring.roles.RoleComposition) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.roles.RoleImplication) {
			result = interprete_org_emftext_language_refactoring_roles_RoleImplication((org.emftext.language.refactoring.roles.RoleImplication) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.roles.RoleAttribute) {
			result = interprete_org_emftext_language_refactoring_roles_RoleAttribute((org.emftext.language.refactoring.roles.RoleAttribute) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.roles.RoleFeature) {
			result = interprete_org_emftext_language_refactoring_roles_RoleFeature((org.emftext.language.refactoring.roles.RoleFeature) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.roles.Commentable) {
			result = interprete_org_emftext_language_refactoring_roles_Commentable((org.emftext.language.refactoring.roles.Commentable) object, context);
		}
		if (result != null) {
			return result;
		}
		return result;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_rolemapping_RoleMappingModel(org.emftext.language.refactoring.rolemapping.RoleMappingModel roleMappingModel, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_rolemapping_RoleMapping(org.emftext.language.refactoring.rolemapping.RoleMapping roleMapping, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_rolemapping_ConcreteMapping(org.emftext.language.refactoring.rolemapping.ConcreteMapping concreteMapping, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_rolemapping_CollaborationMapping(org.emftext.language.refactoring.rolemapping.CollaborationMapping collaborationMapping, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_rolemapping_AttributeMapping(org.emftext.language.refactoring.rolemapping.AttributeMapping attributeMapping, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_rolemapping_ReferenceMetaClassPair(org.emftext.language.refactoring.rolemapping.ReferenceMetaClassPair referenceMetaClassPair, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_EAttribute(org.eclipse.emf.ecore.EAttribute eAttribute, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_EAnnotation(org.eclipse.emf.ecore.EAnnotation eAnnotation, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_EClass(org.eclipse.emf.ecore.EClass eClass, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_EClassifier(org.eclipse.emf.ecore.EClassifier eClassifier, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_EDataType(org.eclipse.emf.ecore.EDataType eDataType, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_EEnum(org.eclipse.emf.ecore.EEnum eEnum, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_EEnumLiteral(org.eclipse.emf.ecore.EEnumLiteral eEnumLiteral, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_EFactory(org.eclipse.emf.ecore.EFactory eFactory, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_EModelElement(org.eclipse.emf.ecore.EModelElement eModelElement, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_ENamedElement(org.eclipse.emf.ecore.ENamedElement eNamedElement, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_EObject(org.eclipse.emf.ecore.EObject eObject, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_EOperation(org.eclipse.emf.ecore.EOperation eOperation, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_EPackage(org.eclipse.emf.ecore.EPackage ePackage, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_EParameter(org.eclipse.emf.ecore.EParameter eParameter, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_EReference(org.eclipse.emf.ecore.EReference eReference, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_EStructuralFeature(org.eclipse.emf.ecore.EStructuralFeature eStructuralFeature, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_ETypedElement(org.eclipse.emf.ecore.ETypedElement eTypedElement, ContextType context) {
		return null;
	}
	
	public ResultType interprete_java_util_Map_Entry(java.util.Map.Entry<?,?> eStringToStringMapEntry, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_EGenericType(org.eclipse.emf.ecore.EGenericType eGenericType, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_eclipse_emf_ecore_ETypeParameter(org.eclipse.emf.ecore.ETypeParameter eTypeParameter, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_roles_NamedElement(org.emftext.language.refactoring.roles.NamedElement namedElement, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_roles_Collaboration(org.emftext.language.refactoring.roles.Collaboration collaboration, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_roles_Role(org.emftext.language.refactoring.roles.Role role, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_roles_RoleModel(org.emftext.language.refactoring.roles.RoleModel roleModel, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_roles_Multiplicity(org.emftext.language.refactoring.roles.Multiplicity multiplicity, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_roles_MultiplicityCollaboration(org.emftext.language.refactoring.roles.MultiplicityCollaboration multiplicityCollaboration, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_roles_RoleProhibition(org.emftext.language.refactoring.roles.RoleProhibition roleProhibition, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_roles_RoleAssociation(org.emftext.language.refactoring.roles.RoleAssociation roleAssociation, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_roles_RoleComposition(org.emftext.language.refactoring.roles.RoleComposition roleComposition, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_roles_RoleImplication(org.emftext.language.refactoring.roles.RoleImplication roleImplication, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_roles_RoleAttribute(org.emftext.language.refactoring.roles.RoleAttribute roleAttribute, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_roles_RoleFeature(org.emftext.language.refactoring.roles.RoleFeature roleFeature, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_roles_Commentable(org.emftext.language.refactoring.roles.Commentable commentable, ContextType context) {
		return null;
	}
	
	private void notifyListeners(org.eclipse.emf.ecore.EObject element) {
		for (org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingInterpreterListener listener : listeners) {
			listener.handleInterpreteObject(element);
		}
	}
	
	/**
	 * Adds the given object to the interpretation stack. Attention: Objects that are
	 * added first, are interpret last.
	 */
	public void addObjectToInterprete(org.eclipse.emf.ecore.EObject object) {
		interpretationStack.push(object);
	}
	
	/**
	 * Adds the given collection of objects to the interpretation stack. Attention:
	 * Collections that are added first, are interpret last.
	 */
	public void addObjectsToInterprete(java.util.Collection<? extends org.eclipse.emf.ecore.EObject> objects) {
		for (org.eclipse.emf.ecore.EObject object : objects) {
			addObjectToInterprete(object);
		}
	}
	
	/**
	 * Adds the given collection of objects in reverse order to the interpretation
	 * stack.
	 */
	public void addObjectsToInterpreteInReverseOrder(java.util.Collection<? extends org.eclipse.emf.ecore.EObject> objects) {
		java.util.List<org.eclipse.emf.ecore.EObject> reverse = new java.util.ArrayList<org.eclipse.emf.ecore.EObject>(objects.size());
		reverse.addAll(objects);
		java.util.Collections.reverse(reverse);
		addObjectsToInterprete(reverse);
	}
	
	/**
	 * Adds the given object and all its children to the interpretation stack such
	 * that they are interpret in top down order.
	 */
	public void addObjectTreeToInterpreteTopDown(org.eclipse.emf.ecore.EObject root) {
		java.util.List<org.eclipse.emf.ecore.EObject> objects = new java.util.ArrayList<org.eclipse.emf.ecore.EObject>();
		objects.add(root);
		java.util.Iterator<org.eclipse.emf.ecore.EObject> it = root.eAllContents();
		while (it.hasNext()) {
			org.eclipse.emf.ecore.EObject eObject = (org.eclipse.emf.ecore.EObject) it.next();
			objects.add(eObject);
		}
		addObjectsToInterpreteInReverseOrder(objects);
	}
	
	public void addListener(org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingInterpreterListener newListener) {
		listeners.add(newListener);
	}
	
	public boolean removeListener(org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingInterpreterListener listener) {
		return listeners.remove(listener);
	}
	
	public org.eclipse.emf.ecore.EObject getNextObjectToInterprete() {
		return nextObjectToInterprete;
	}
	
	public java.util.Stack<org.eclipse.emf.ecore.EObject> getInterpretationStack() {
		return interpretationStack;
	}
	
	public void terminate() {
		interpretationStack.clear();
	}
	
	public Object getCurrentContext() {
		return currentContext;
	}
	
}
