/*******************************************************************************
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 ******************************************************************************/

package org.emftext.language.refactoring.specification.resource.util;

/**
 * This class provides basic infrastructure to interpret models. To implement
 * concrete interpreters, subclass this abstract interpreter and override the
 * interprete_* methods. The interpretation can be customized by binding the two
 * type parameters (ResultType, ContextType). The former is returned by all
 * interprete_* methods, while the latter is passed from method to method while
 * traversing the model. The concrete traversal strategy can also be exchanged.
 * One can use a static traversal strategy by pushing all objects to interpret on
 * the interpretation stack (using addObjectToInterprete()) before calling
 * interprete(). Alternatively, the traversal strategy can be dynamic by pushing
 * objects on the interpretation stack during interpretation.
 */
public class AbstractRefspecInterpreter<ResultType, ContextType> {
	
	private java.util.Stack<org.eclipse.emf.ecore.EObject> interpretationStack = new java.util.Stack<org.eclipse.emf.ecore.EObject>();
	private java.util.List<org.emftext.language.refactoring.specification.resource.IRefspecInterpreterListener> listeners = new java.util.ArrayList<org.emftext.language.refactoring.specification.resource.IRefspecInterpreterListener>();
	private org.eclipse.emf.ecore.EObject nextObjectToInterprete;
	private Object currentContext;
	
	public ResultType interprete(ContextType context) {
		ResultType result = null;
		org.eclipse.emf.ecore.EObject next = null;
		currentContext = context;
		while (!interpretationStack.empty()) {
			try {
				next = interpretationStack.pop();
			} catch (java.util.EmptyStackException ese) {
				// this can happen when the interpreter was terminated between the call to empty()
				// and pop()
				break;
			}
			nextObjectToInterprete = next;
			notifyListeners(next);
			result = interprete(next, context);
			if (!continueInterpretation(context, result)) {
				break;
			}
		}
		currentContext = null;
		return result;
	}
	
	/**
	 * Override this method to stop the overall interpretation depending on the result
	 * of the interpretation of a single model elements.
	 */
	public boolean continueInterpretation(ContextType context, ResultType result) {
		return true;
	}
	
	public ResultType interprete(org.eclipse.emf.ecore.EObject object, ContextType context) {
		ResultType result = null;
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.RefactoringSpecification) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_RefactoringSpecification((org.emftext.language.refactoring.refactoring_specification.RefactoringSpecification) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.CREATE) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_CREATE((org.emftext.language.refactoring.refactoring_specification.CREATE) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.MOVE) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_MOVE((org.emftext.language.refactoring.refactoring_specification.MOVE) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.ContainmentCommand) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_ContainmentCommand((org.emftext.language.refactoring.refactoring_specification.ContainmentCommand) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.Instruction) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_Instruction((org.emftext.language.refactoring.refactoring_specification.Instruction) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.Variable) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_Variable((org.emftext.language.refactoring.refactoring_specification.Variable) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.VariableReference) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_VariableReference((org.emftext.language.refactoring.refactoring_specification.VariableReference) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.SourceContext) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_SourceContext((org.emftext.language.refactoring.refactoring_specification.SourceContext) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.RoleReference) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_RoleReference((org.emftext.language.refactoring.refactoring_specification.RoleReference) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.CollaborationReference) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_CollaborationReference((org.emftext.language.refactoring.refactoring_specification.CollaborationReference) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.TargetContext) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_TargetContext((org.emftext.language.refactoring.refactoring_specification.TargetContext) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.SET) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_SET((org.emftext.language.refactoring.refactoring_specification.SET) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.ReferenceCommand) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_ReferenceCommand((org.emftext.language.refactoring.refactoring_specification.ReferenceCommand) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.ASSIGN) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_ASSIGN((org.emftext.language.refactoring.refactoring_specification.ASSIGN) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.AttributeCommand) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_AttributeCommand((org.emftext.language.refactoring.refactoring_specification.AttributeCommand) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.FromClause) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_FromClause((org.emftext.language.refactoring.refactoring_specification.FromClause) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.UPTREE) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_UPTREE((org.emftext.language.refactoring.refactoring_specification.UPTREE) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.FromOperator) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_FromOperator((org.emftext.language.refactoring.refactoring_specification.FromOperator) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.ConstantsReference) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_ConstantsReference((org.emftext.language.refactoring.refactoring_specification.ConstantsReference) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.ObjectReference) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_ObjectReference((org.emftext.language.refactoring.refactoring_specification.ObjectReference) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.Context) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_Context((org.emftext.language.refactoring.refactoring_specification.Context) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.IndexAssignmentCommand) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_IndexAssignmentCommand((org.emftext.language.refactoring.refactoring_specification.IndexAssignmentCommand) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.IndexVariable) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_IndexVariable((org.emftext.language.refactoring.refactoring_specification.IndexVariable) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.ConcreteIndex) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_ConcreteIndex((org.emftext.language.refactoring.refactoring_specification.ConcreteIndex) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.FIRST) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_FIRST((org.emftext.language.refactoring.refactoring_specification.FIRST) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.LAST) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_LAST((org.emftext.language.refactoring.refactoring_specification.LAST) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.ObjectReferenceAssignment) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_ObjectReferenceAssignment((org.emftext.language.refactoring.refactoring_specification.ObjectReferenceAssignment) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.ObjectAssignmentCommand) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_ObjectAssignmentCommand((org.emftext.language.refactoring.refactoring_specification.ObjectAssignmentCommand) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.VariableDeclarationCommand) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_VariableDeclarationCommand((org.emftext.language.refactoring.refactoring_specification.VariableDeclarationCommand) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.TRACE) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_TRACE((org.emftext.language.refactoring.refactoring_specification.TRACE) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.TraceObject) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_TraceObject((org.emftext.language.refactoring.refactoring_specification.TraceObject) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.VariableAssignment) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_VariableAssignment((org.emftext.language.refactoring.refactoring_specification.VariableAssignment) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.PATH) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_PATH((org.emftext.language.refactoring.refactoring_specification.PATH) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.FILTER) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_FILTER((org.emftext.language.refactoring.refactoring_specification.FILTER) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.DISTINCT) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_DISTINCT((org.emftext.language.refactoring.refactoring_specification.DISTINCT) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.Modifier) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_Modifier((org.emftext.language.refactoring.refactoring_specification.Modifier) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.UNSET) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_UNSET((org.emftext.language.refactoring.refactoring_specification.UNSET) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.REMOVE) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_REMOVE((org.emftext.language.refactoring.refactoring_specification.REMOVE) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.DeleteCommand) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_DeleteCommand((org.emftext.language.refactoring.refactoring_specification.DeleteCommand) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.RoleRemoval) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_RoleRemoval((org.emftext.language.refactoring.refactoring_specification.RoleRemoval) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.ObjectRemoval) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_ObjectRemoval((org.emftext.language.refactoring.refactoring_specification.ObjectRemoval) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.UNUSED) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_UNUSED((org.emftext.language.refactoring.refactoring_specification.UNUSED) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.EMPTY) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_EMPTY((org.emftext.language.refactoring.refactoring_specification.EMPTY) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.RemoveModifier) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_RemoveModifier((org.emftext.language.refactoring.refactoring_specification.RemoveModifier) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.AFTER) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_AFTER((org.emftext.language.refactoring.refactoring_specification.AFTER) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.refactoring.refactoring_specification.AbstractVariable) {
			result = interprete_org_emftext_language_refactoring_refactoring_005fspecification_AbstractVariable((org.emftext.language.refactoring.refactoring_specification.AbstractVariable) object, context);
		}
		if (result != null) {
			return result;
		}
		return result;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_RefactoringSpecification(org.emftext.language.refactoring.refactoring_specification.RefactoringSpecification refactoringSpecification, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_Instruction(org.emftext.language.refactoring.refactoring_specification.Instruction instruction, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_ContainmentCommand(org.emftext.language.refactoring.refactoring_specification.ContainmentCommand containmentCommand, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_CREATE(org.emftext.language.refactoring.refactoring_specification.CREATE cREATE, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_MOVE(org.emftext.language.refactoring.refactoring_specification.MOVE mOVE, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_Variable(org.emftext.language.refactoring.refactoring_specification.Variable variable, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_VariableReference(org.emftext.language.refactoring.refactoring_specification.VariableReference variableReference, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_SourceContext(org.emftext.language.refactoring.refactoring_specification.SourceContext sourceContext, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_RoleReference(org.emftext.language.refactoring.refactoring_specification.RoleReference roleReference, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_CollaborationReference(org.emftext.language.refactoring.refactoring_specification.CollaborationReference collaborationReference, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_TargetContext(org.emftext.language.refactoring.refactoring_specification.TargetContext targetContext, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_ReferenceCommand(org.emftext.language.refactoring.refactoring_specification.ReferenceCommand referenceCommand, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_SET(org.emftext.language.refactoring.refactoring_specification.SET sET, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_AttributeCommand(org.emftext.language.refactoring.refactoring_specification.AttributeCommand attributeCommand, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_ASSIGN(org.emftext.language.refactoring.refactoring_specification.ASSIGN aSSIGN, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_FromClause(org.emftext.language.refactoring.refactoring_specification.FromClause fromClause, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_FromOperator(org.emftext.language.refactoring.refactoring_specification.FromOperator fromOperator, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_UPTREE(org.emftext.language.refactoring.refactoring_specification.UPTREE uPTREE, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_ConstantsReference(org.emftext.language.refactoring.refactoring_specification.ConstantsReference constantsReference, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_ObjectReference(org.emftext.language.refactoring.refactoring_specification.ObjectReference objectReference, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_Context(org.emftext.language.refactoring.refactoring_specification.Context _context, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_IndexAssignmentCommand(org.emftext.language.refactoring.refactoring_specification.IndexAssignmentCommand indexAssignmentCommand, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_IndexVariable(org.emftext.language.refactoring.refactoring_specification.IndexVariable indexVariable, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_ConcreteIndex(org.emftext.language.refactoring.refactoring_specification.ConcreteIndex concreteIndex, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_FIRST(org.emftext.language.refactoring.refactoring_specification.FIRST fIRST, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_LAST(org.emftext.language.refactoring.refactoring_specification.LAST lAST, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_ObjectReferenceAssignment(org.emftext.language.refactoring.refactoring_specification.ObjectReferenceAssignment objectReferenceAssignment, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_ObjectAssignmentCommand(org.emftext.language.refactoring.refactoring_specification.ObjectAssignmentCommand objectAssignmentCommand, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_VariableDeclarationCommand(org.emftext.language.refactoring.refactoring_specification.VariableDeclarationCommand variableDeclarationCommand, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_TRACE(org.emftext.language.refactoring.refactoring_specification.TRACE tRACE, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_TraceObject(org.emftext.language.refactoring.refactoring_specification.TraceObject traceObject, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_VariableAssignment(org.emftext.language.refactoring.refactoring_specification.VariableAssignment variableAssignment, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_PATH(org.emftext.language.refactoring.refactoring_specification.PATH pATH, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_FILTER(org.emftext.language.refactoring.refactoring_specification.FILTER fILTER, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_Modifier(org.emftext.language.refactoring.refactoring_specification.Modifier modifier, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_DISTINCT(org.emftext.language.refactoring.refactoring_specification.DISTINCT dISTINCT, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_DeleteCommand(org.emftext.language.refactoring.refactoring_specification.DeleteCommand deleteCommand, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_UNSET(org.emftext.language.refactoring.refactoring_specification.UNSET uNSET, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_REMOVE(org.emftext.language.refactoring.refactoring_specification.REMOVE rEMOVE, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_ObjectRemoval(org.emftext.language.refactoring.refactoring_specification.ObjectRemoval objectRemoval, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_RoleRemoval(org.emftext.language.refactoring.refactoring_specification.RoleRemoval roleRemoval, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_RemoveModifier(org.emftext.language.refactoring.refactoring_specification.RemoveModifier removeModifier, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_UNUSED(org.emftext.language.refactoring.refactoring_specification.UNUSED uNUSED, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_EMPTY(org.emftext.language.refactoring.refactoring_specification.EMPTY eMPTY, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_AFTER(org.emftext.language.refactoring.refactoring_specification.AFTER aFTER, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_refactoring_refactoring_005fspecification_AbstractVariable(org.emftext.language.refactoring.refactoring_specification.AbstractVariable abstractVariable, ContextType context) {
		return null;
	}
	
	private void notifyListeners(org.eclipse.emf.ecore.EObject element) {
		for (org.emftext.language.refactoring.specification.resource.IRefspecInterpreterListener listener : listeners) {
			listener.handleInterpreteObject(element);
		}
	}
	
	/**
	 * Adds the given object to the interpretation stack. Attention: Objects that are
	 * added first, are interpret last.
	 */
	public void addObjectToInterprete(org.eclipse.emf.ecore.EObject object) {
		interpretationStack.push(object);
	}
	
	/**
	 * Adds the given collection of objects to the interpretation stack. Attention:
	 * Collections that are added first, are interpret last.
	 */
	public void addObjectsToInterprete(java.util.Collection<? extends org.eclipse.emf.ecore.EObject> objects) {
		for (org.eclipse.emf.ecore.EObject object : objects) {
			addObjectToInterprete(object);
		}
	}
	
	/**
	 * Adds the given collection of objects in reverse order to the interpretation
	 * stack.
	 */
	public void addObjectsToInterpreteInReverseOrder(java.util.Collection<? extends org.eclipse.emf.ecore.EObject> objects) {
		java.util.List<org.eclipse.emf.ecore.EObject> reverse = new java.util.ArrayList<org.eclipse.emf.ecore.EObject>(objects.size());
		reverse.addAll(objects);
		java.util.Collections.reverse(reverse);
		addObjectsToInterprete(reverse);
	}
	
	/**
	 * Adds the given object and all its children to the interpretation stack such
	 * that they are interpret in top down order.
	 */
	public void addObjectTreeToInterpreteTopDown(org.eclipse.emf.ecore.EObject root) {
		java.util.List<org.eclipse.emf.ecore.EObject> objects = new java.util.ArrayList<org.eclipse.emf.ecore.EObject>();
		objects.add(root);
		java.util.Iterator<org.eclipse.emf.ecore.EObject> it = root.eAllContents();
		while (it.hasNext()) {
			org.eclipse.emf.ecore.EObject eObject = (org.eclipse.emf.ecore.EObject) it.next();
			objects.add(eObject);
		}
		addObjectsToInterpreteInReverseOrder(objects);
	}
	
	public void addListener(org.emftext.language.refactoring.specification.resource.IRefspecInterpreterListener newListener) {
		listeners.add(newListener);
	}
	
	public boolean removeListener(org.emftext.language.refactoring.specification.resource.IRefspecInterpreterListener listener) {
		return listeners.remove(listener);
	}
	
	public org.eclipse.emf.ecore.EObject getNextObjectToInterprete() {
		return nextObjectToInterprete;
	}
	
	public java.util.Stack<org.eclipse.emf.ecore.EObject> getInterpretationStack() {
		return interpretationStack;
	}
	
	public void terminate() {
		interpretationStack.clear();
	}
	
	public Object getCurrentContext() {
		return currentContext;
	}
	
}
